/*
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.mobicents.servlet.sip.pbx;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mobicents.servlet.sip.pbx.address.AddressService;
import org.mobicents.servlet.sip.pbx.db.DbAddressService;
import org.mobicents.servlet.sip.pbx.db.DbLocationService;
import org.mobicents.servlet.sip.pbx.db.DbRouteService;
import org.mobicents.servlet.sip.pbx.location.LocationService;
import org.mobicents.servlet.sip.pbx.routing.RouteService;

/**
 * @author Thomas Leseney
 */
public class PbxListener implements ServletContextListener {

    private static Log logger = LogFactory.getLog(PbxListener.class);

    public void contextInitialized(ServletContextEvent event) {
        logger.info("Initializing PBX context");
        try {
            startLocationService(event);
            startAddressService(event);
            startRouteService(event);
            logger.info("Initialized PBX context");

        } catch (Exception e) {
            logger.warn("Failed to initialize PBX context", e);
        }
    }

    public void contextDestroyed(ServletContextEvent event) {
        logger.info("Finalizing PBX context");
        try {
            stopLocationService(event);
            stopAddressService(event);
            stopRouteService(event);
            logger.info("PBX context finalized");

        } catch (Exception e) {
            logger.error("Failed to finalize PBX context");
        }
    }

    private void startLocationService(ServletContextEvent event) throws Exception {
        LocationService locationService = new DbLocationService();
        locationService.start();
        event.getServletContext().setAttribute(Constants.LOCATION_SERVICE, locationService);
    }

    private void startAddressService(ServletContextEvent event) throws Exception {
        AddressService addressService = new DbAddressService();
        addressService.start();
        event.getServletContext().setAttribute(Constants.ADDRESS_SERVICE, addressService);
    }

    private void startRouteService(ServletContextEvent event) throws Exception {
        RouteService routeService = new DbRouteService();
        routeService.start();
        event.getServletContext().setAttribute(Constants.ROUTE_SERVICE, routeService);
    }

    private void stopLocationService(ServletContextEvent event) throws Exception {
        LocationService locationService = (LocationService) event.getServletContext().getAttribute(Constants.LOCATION_SERVICE);
        if (locationService != null) {
            locationService.stop();
        }
    }

    private void stopAddressService(ServletContextEvent event) throws Exception {
        AddressService addressService = (AddressService) event.getServletContext().getAttribute(Constants.ADDRESS_SERVICE);
        if (addressService != null) {
            addressService.stop();
        }
    }

    private void stopRouteService(ServletContextEvent event) throws Exception {
        RouteService routeService = (RouteService) event.getServletContext().getAttribute(Constants.ROUTE_SERVICE);
        if (routeService != null) {
            routeService.stop();
        }
    }
}
