/*
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.mobicents.servlet.sip.pbx.db;

import javax.servlet.sip.SipURI;
import javax.servlet.sip.URI;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.mobicents.servlet.sip.pbx.address.Address;
import org.mobicents.servlet.sip.pbx.address.BaseAddressService;

/**
 * @author Marcos Hack <marcoshack@gmail.com>
 */
public class DbAddressService extends BaseAddressService {

    public void start() throws Exception {
        HibernateUtil.getSessionFactory();
    }

    public void stop() throws Exception {
        HibernateUtil.getSessionFactory().close();
    }

    private Session getSession() {
        return HibernateUtil.getSessionFactory().getCurrentSession();
    }

    public void beginTransaction() {
        getSession().beginTransaction();
    }

    public void commitTransaction() {
        Transaction transaction = getSession().getTransaction();
        if (transaction != null && transaction.isActive()) {
            getSession().getTransaction().commit();
        }
    }

    public void rollbackTransaction() {
        getSession().getTransaction().rollback();
    }

    public String getAddressOfRecordByStation(URI uri) {
        beginTransaction();
        Query query = getSession().createQuery("FROM Address WHERE number = :number");
        query.setParameter("number", ((SipURI) uri).getUser());

        try {
            Address address = (Address) query.uniqueResult();
            if (address != null) {
                return getAddressOfRecord(address.getUser());
            } else {
                return null;
            }

        } finally {
            commitTransaction();
        }
    }
}
