/*
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.mobicents.servlet.sip.pbx.db;

import java.util.Iterator;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.mobicents.servlet.sip.pbx.location.Binding;
import org.mobicents.servlet.sip.pbx.location.LocationService;

/**
 * @author Thomas Leseney
 */
public class DbLocationService implements LocationService {

    private Timer timer;

    public DbLocationService() {
    }

    public void start() throws Exception {
        HibernateUtil.getSessionFactory();
        timer = new Timer();
        timer.schedule(new Scavenger(), 5000, 5000);
    }

    public void stop() throws Exception {
        timer.cancel();
        HibernateUtil.getSessionFactory().close();
    }

    private Session getSession() {
        Session session = HibernateUtil.getSessionFactory().getCurrentSession();
        session.beginTransaction();
        return session;
    }

    public Binding createBinding(String aor, String contact) {
        return new Binding(aor, contact);
    }

    public List<Binding> getBindings(String aor) {
        try {
            return getSession().createQuery("FROM Binding b where b.aor=:aor").setParameter("aor", aor).list();
        } finally {
            commitTransaction();
        }
    }

    public List<Binding> getAllBindings() {
        try {
            return getSession().createQuery("FROM Binding b").list();
        } finally {
            commitTransaction();
        }
    }

    public void addBinding(Binding binding) {
        try {
            getSession().save(binding);
        } finally {
            commitTransaction();
        }
    }

    public void updateBinding(Binding binding) {
    }

    public void removeBinding(Binding binding) {
        try {
            getSession().delete(binding);
        } finally {
            commitTransaction();
        }
    }

    public void removeExpiredBindings() {
        try {
            Session session = getSession();
            Query query = session.createQuery("FROM Binding WHERE expirationTime < :now");
            query.setParameter("now", System.currentTimeMillis());
            Iterator<Binding> it = query.iterate();
            while (it.hasNext()) {
                Binding binding = (Binding) it.next();
                session.delete(binding);
            }
        } finally {
            commitTransaction();
        }
    }

    public void commitTransaction() {
        Transaction transaction = getSession().getTransaction();
        if (transaction != null && transaction.isActive()) {
            getSession().getTransaction().commit();
        }
    }

    public void rollbackTransaction() {
        Transaction transaction = getSession().getTransaction();
        if (transaction != null && transaction.isActive()) {
            getSession().getTransaction().rollback();
        }
    }

    class Scavenger extends TimerTask {
        public void run() {
            try {
                removeExpiredBindings();
            } finally {
                commitTransaction();
            }
        }
    }
}
