package org.mobicents.servlet.sip.pbx.db;

import java.util.List;
import javax.servlet.sip.SipURI;
import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.mobicents.servlet.sip.pbx.routing.Route;
import org.mobicents.servlet.sip.pbx.routing.RouteService;

/**
 *
 * @author Marcos Hack <marcoshack@gmail.com>
 */
public class DbRouteService implements RouteService {

    private Logger logger = Logger.getLogger(DbRouteService.class);

    public void start() throws Exception {
        HibernateUtil.getSessionFactory();
    }

    public void stop() throws Exception {
        HibernateUtil.getSessionFactory().close();
    }

    protected Session getSession() {
        Session session = HibernateUtil.getSessionFactory().getCurrentSession();
        session.beginTransaction();
        return session;
    }

    public void commitTransaction() {
        Transaction transaction = getSession().getTransaction();
        if (transaction != null && transaction.isActive()) {
            getSession().getTransaction().commit();
        }
    }

    public void rollbackTransaction() {
        getSession().getTransaction().rollback();
    }

    public Route getRoute(SipURI uri) {
        try {
            List<Route> routes = getSession().createQuery("FROM Route").list();

            if (logger.isDebugEnabled()) {
                StringBuilder sb = new StringBuilder("Route list: ").append(routes);
                logger.debug(sb);
            }

            for (Route route : routes) {
                if (uri.getUser().startsWith(route.getPrefix())) {
                    return route;
                }
            }
            return null;
            
        } finally {
            commitTransaction();
        }
    }
}
