package org.mobicents.servlet.sip.pbx.db;

import java.util.ArrayList;
import java.util.List;
import org.hibernate.Query;
import org.hibernate.Session;
import org.junit.Test;
import org.mobicents.servlet.sip.pbx.address.SipURIMock;
import org.mobicents.servlet.sip.pbx.routing.Gateway;
import org.mobicents.servlet.sip.pbx.routing.Route;
import static org.mockito.Mockito.*;
import static org.junit.Assert.*;

/**
 *
 * @author Marcos Hack <marcoshack@gmail.com>
 */
public class DbRouteServiceTest {

    @Test
    public void testGetRoute() {
        Gateway gateway1 = new Gateway("127.0.0.1:5060");
        Gateway gateway2 = new Gateway("127.0.0.1:5080");
        List routeList = new ArrayList();
        routeList.add(new Route("5521", gateway2));
        routeList.add(new Route("5511", gateway1));
        DbRouteService routeService = new DbRouteServiceMock(routeList);

        Route route1 = routeService.getRoute(new SipURIMock("551196413181", "example.net"));
        assertNotNull(route1);
        assertEquals(gateway1.getAddress(), route1.getGateway().getAddress());

        Route route2 = routeService.getRoute(new SipURIMock("552196413181", "example.net"));
        assertNotNull(route2);
        assertEquals(gateway2.getAddress(), route2.getGateway().getAddress());
    }

    public class DbRouteServiceMock extends DbRouteService {

        private List<Route> routeList;

        public DbRouteServiceMock(List<Route> routeList) {
            this.routeList = routeList;
        }

        @Override
        protected Session getSession() {
            Query query = mock(Query.class);
            when(query.list()).thenReturn(routeList);
            Session session = mock(Session.class);
            when(session.createQuery("FROM Route")).thenReturn(query);
            return session;
        }
    }
}
